pub(crate) struct DevtoolsConnection {
receiver: Arc<Mutex<TcpStream>>,
sender: Arc<Mutex<TcpStream>>,
}Expand description
Wraps a Remote Debugging Protocol TCP stream, guaranteeing that network operations are synchronized when cloning across threads.
Fields
receiver: Arc<Mutex<TcpStream>>Copy of TcpStream handle to use for receiving from the client.
TcpStream::read is a mutating I/O operation that doesn’t fit with RwLock.
We clone a single stream handle into two mutexes so we can still lock
reads and writes independently.
sender: Arc<Mutex<TcpStream>>Copy of TcpStream handle to use for sending bytes to the client.
Implementations
Sourceimpl DevtoolsConnection
impl DevtoolsConnection
Sourcepub(crate) fn peer_addr(&self) -> Result<SocketAddr>
pub(crate) fn peer_addr(&self) -> Result<SocketAddr>
Calls TcpStream::peer_addr on the underlying stream.